import { LoadCreationAction } from "@haulynx/entities";
import { missionManagementApiClient } from "../api-clients/MissionManagementApiClient";
import { usxDataBridgeClient } from "../api-clients/UsxDataBridgeClient";
import { logger } from "../services";
import format from "date-fns-tz/format";
import { getServiceToServiceToken } from "../api-clients";
import { MISSION_MANAGEMENT_API_URL } from "../conifg";

const formatDateOrderCreatePayload = (epochTime: number): string =>
  new Date(epochTime).toISOString().replace(/T/, "-").replace(/:/g, ".").replace(/Z/, "000");

const buildUsxOrderCreatePayloadFromLoadCreationAction = (payload: any, loadCreationAction: LoadCreationAction): any => {
  const originEarlyDateTimeEpoch = Date.parse(payload.last_stop.late_appointment) + loadCreationAction.originAppointmentTimeOffset * 60000;
  const originLateDateTimeEpoch = originEarlyDateTimeEpoch + loadCreationAction.originAppointmentTimeInterval * 60000;
  const destinationEarlyDateTimeEpoch = originLateDateTimeEpoch + loadCreationAction.destinationAppointmentTimeOffset * 60000;
  const destinationLateDateTimeEpoch = destinationEarlyDateTimeEpoch + loadCreationAction.destinationAppointmentTimeInterval * 60000;

  return {
    auditUser: "XTSYSTEM",
    companyNumber: "01",
    orderDate: format(new Date(), "yyyy-MM-dd"),

    billToCustomerId: Number(loadCreationAction.billTo),
    shipperCustomerId: Number(loadCreationAction.originCustomer),
    consigneeCustomerId: Number(loadCreationAction.destinationCustomer),

    liveLoadDropLoad: "D",
    originLoadType: payload.unload_requirements === "L" ? "L" : "P" ,

    originEarlyDateTime: formatDateOrderCreatePayload(originEarlyDateTimeEpoch), // YYYY-MM-DD-HH.MM.SS.ssssss - Chattanooga time
    originLoadUnloadTime: "2.0",
    originLateDateTime: formatDateOrderCreatePayload(originLateDateTimeEpoch), // YYYY-MM-DD-HH.MM.SS.ssssss - Chattanooga time
    originAppointmentSet: "Y",

    destinEarlyDateTime: formatDateOrderCreatePayload(destinationEarlyDateTimeEpoch), // YYYY-MM-DD-HH.MM.SS.ssssss - Chattanooga time
    destinLoadUnloadTime: "2.0",
    destinLateDateTime: formatDateOrderCreatePayload(destinationLateDateTimeEpoch), // YYYY-MM-DD-HH.MM.SS.ssssss - Chattanooga time
    destinAppointmentSet: "Y",

    teamRequired: "N",
    teamPayrate: "N",
    driverLoad: "N",
    driverUnload: "N",
    hazmatLoad: "N",
    headerLine: [
      {
        lineType: "LN",
        description: "FAK"
      }
    ]
  };
};

export const processOrderChainingEvent = () => async (data: any): Promise<void> => {
  const location = "processOrderChainingEvent";

  const { meta, payload } = JSON.parse(data.message.value?.toString());

  if (meta.intent !== "event-created" && meta.intent !== "event-upserted") {
    return;
  }

  logger.trace({
    location,
    payload
  });


  const { bill_to, first_stop, last_stop } = payload;

  // if source order's appointments are not set, do not process it.
  if (!first_stop.appointment_set || !last_stop.appointment_set) {
    return;
  }

  const token = await getServiceToServiceToken(MISSION_MANAGEMENT_API_URL);
  missionManagementApiClient.setToken(token);

  // if there is already a created related load, do not create a new one.
  const existingRelatedLoadCreation = await missionManagementApiClient.searchRelatedLoadCreation({ sourceOrderNumber: String(payload.order_number) });
  if (existingRelatedLoadCreation?.length) {
    return;
  }

  const laneRules = await missionManagementApiClient.searchLaneRule({
      billTo: bill_to,
      originCustomer: first_stop?.stop_customer,
      destinationCustomer: last_stop?.stop_customer
  });

  if (!laneRules || !laneRules.length) {
    return;
  }

  logger.trace({
    location,
    message: "Lane rule applicable order",
    laneRules
  });

  const orderCreatePayload = buildUsxOrderCreatePayloadFromLoadCreationAction(payload, laneRules[0].loadCreationAction);
  logger.trace({
    location,
    message: "sendOrderCreatePayload",
    orderCreatePayload
  });

  let response, orderNumber;
  try {
    ({ response, orderNumber } = await usxDataBridgeClient.sendOrderCreate(orderCreatePayload));
  } catch (error) {
    logger.error({
      location,
      orderNumber: payload.order_number,
      error
    });
    throw error;
  }

  /**
   * TODO: if order creation is success, create a new relatedLoadCreation for mission creation
   * For testing, see if new dummy order that satisfies the test rule created an entry.
   */

  const relatedLoadCreation = await missionManagementApiClient.createRelatedLoadCreation({
    sourceOrderNumber: String(payload.order_number),
    relatedOrderNumber: orderNumber,
    loadCreationActionId: laneRules[0].loadCreationAction.id
  });

  logger.trace({
    location,
    message: "Related Load Creation Created",
    relatedLoadCreation,
    source: {
      billTo: bill_to,
      originCustomer: first_stop?.stop_customer,
      destinationCustomer: last_stop?.stop_customer
    }
  });
};
